# USAGE
# python detect_mask_image.py --image examples/example_01.png

# import the necessary packages
from tensorflow.keras.applications.mobilenet_v2 import preprocess_input
from tensorflow.keras.preprocessing.image import img_to_array
from tensorflow.keras.models import load_model
import numpy as np
import argparse
import cv2
import os

# bagian ini untuk mengimport image inputan dari terminal menggunakan path --image===================
# construct the argument parser and parse the arguments
ap = argparse.ArgumentParser()
ap.add_argument("-i", "--image", required=True,
	help="path to input image")
ap.add_argument("-f", "--face", type=str,
	default="face_detector",
	help="path to face detector model directory")
ap.add_argument("-m", "--model", type=str,
	default="mask_detector.model",
	help="path to trained face mask detector model")
ap.add_argument("-c", "--confidence", type=float, default=0.5,
	help="minimum probability to filter weak detections")
args = vars(ap.parse_args())
# ========================================================================================

# bagian ini untuk membaca deteksi wajah dari disk dengan cv2.dnn.readNet=================
# load our serialized face detector model from disk
print("[INFO] loading face detector model...")
prototxtPath = os.path.sep.join([args["face"], "deploy.prototxt"])
weightsPath = os.path.sep.join([args["face"],
	"res10_300x300_ssd_iter_140000.caffemodel"])
net = cv2.dnn.readNet(prototxtPath, weightsPath)
# =======================================================================================

# untuk mendeteksi masker wajah dari disk yang akan dimasukkan ke convolutional neural network/jaringan syaraf tiruan konvolusi==========
# load the face mask detector model from disk
print("[INFO] loading face mask detector model...")
model = load_model(args["model"])
# ===================================================================================================

# menginput gambar dari disk dengan cv2 dan disalin==========================================
# load the input image from disk, clone it, and grab the image spatial
# dimensions
image = cv2.imread(args["image"])
orig = image.copy()
(h, w) = image.shape[:2]
# ==============================================================================================

# blob itu representasi dari gambar input yang dimasukkan ke dalam network=======================
# construct a blob from the image
blob = cv2.dnn.blobFromImage(image, 1.0, (300, 300),
	(104.0, 177.0, 123.0))
# ================================================================================================

# blob gambar yang sudah diproses tadi dimasukkan ke model deteksi wajah dengan forward====================
# pass the blob through the network and obtain the face detections
print("[INFO] computing face detections...")
net.setInput(blob)
detections = net.forward()
# ==========================================================================================================

# looping untuk deteksi wajah dan nilai kepercayaanya akan diekstrak, jika kepercayaan di bawah nilai argumen bakal dilewatkan=========
# loop over the detections
for i in range(0, detections.shape[2]):
	# extract the confidence (i.e., probability) associated with
	# the detection
	confidence = detections[0, 0, i, 2]

	# filter out weak detections by ensuring the confidence is
	# greater than the minimum confidence
	if confidence > args["confidence"]:
# =================================================================================================================

# disini akan dibuat kotak pembatas berisi wajah yang terdeteksi======================================== 
		# compute the (x, y)-coordinates of the bounding box for
		# the object
		box = detections[0, 0, i, 3:7] * np.array([w, h, w, h])
		(startX, startY, endX, endY) = box.astype("int")

		# ensure the bounding boxes fall within the dimensions of
		# the frame
		(startX, startY) = (max(0, startX), max(0, startY))
		(endX, endY) = (min(w - 1, endX), min(h - 1, endY))
# =================================================================================================================

# disini bakal ditampilkan hasil dari image yang berisi wajah yang terdeteksi tadi===========================
		# extract the face ROI, convert it from BGR to RGB channel
		# ordering, resize it to 224x224, and preprocess it
		face = image[startY:endY, startX:endX]
		face = cv2.cvtColor(face, cv2.COLOR_BGR2RGB)
		face = cv2.resize(face, (224, 224)) #image hasilnya dibuat dalam ukuran 224X224
		face = img_to_array(face)
		face = preprocess_input(face)
		face = np.expand_dims(face, axis=0)
# ===================================================================================================================

# wajah yang udah terdeteksi tadi bakal dimasukin ke dalam model deteksi masker======================================
		# pass the face through the model to determine if the face
		# has a mask or not
		(mask, withoutMask) = model.predict(face)[0]

		# determine the class label and color we'll use to draw
		# the bounding box and text
		label = "Masker" if mask > withoutMask else "No Masker" #bakal ditentuin pakek masker atau tidak
		color = (0, 255, 0) if label == "Masker" else (0, 0, 255) #kalau tidak bounding boxnya merah, kalau ya bounding boxnya hijau===========

		# include the probability in the label
		label = "{}: {:.2f}%".format(label, max(mask, withoutMask) * 100)
# ===============================================================================================================================================
		
# hasil deteksi masker dan label tadi bakal ditambahin ke image asli yang bukan salinan tadi========================
		# display the label and bounding box rectangle on the output
		# frame
		cv2.putText(image, label, (startX, startY - 10),
			cv2.FONT_HERSHEY_SIMPLEX, 0.45, color, 2)
		cv2.rectangle(image, (startX, startY), (endX, endY), color, 2)
# =======================================================================================================================

# hasilnya bakal ditampilin ke window baru
# show the output image
cv2.imshow("Output", image)
cv2.waitKey(0)